searchNodes=[{"doc":"Mox is a library for defining concurrent mocks in Elixir.The library follows the principles outlined in &quot;Mocks and explicit contracts&quot;, summarized below:No ad-hoc mocks. You can only create mocks based on behavioursNo dynamic generation of modules during tests. Mocks are preferably defined in your test_helper.exs or in a setup_all block and not per testConcurrency support. Tests using the same mock can still use async: trueRely on pattern matching and function clauses for asserting on the input instead of complex expectation rulesExampleAs an example, imagine that your project defines a WeatherAPI behaviour:defmodule MyApp.WeatherAPI do @callback temp(MyApp.LatLong.t()) :: {:ok, integer()} @callback humidity(MyApp.LatLong.t()) :: {:ok, integer()} endIf you want to mock the WeatherAPI behaviour during tests, the first step is to define the mock with defmock/2, usually in your test_helper.exs:Mox.defmock(MyApp.MockWeatherAPI, for: MyApp.WeatherAPI)Now in your tests, you can define expectations with expect/4 and verify them via verify_on_exit!/1:defmodule MyApp.HumanizedWeatherTest do use ExUnit.Case, async: true import Mox # Make sure mocks are verified when the test exits setup :verify_on_exit! test &quot;gets and formats temperature and humidity&quot; do MyApp.MockWeatherAPI |&gt; expect(:temp, fn {_lat, _long} -&gt; {:ok, 30} end) |&gt; expect(:humidity, fn {_lat, _long} -&gt; {:ok, 60} end) assert MyApp.HumanizedWeather.temp({50.06, 19.94}) == &quot;Current temperature is 30 degrees&quot; assert MyApp.HumanizedWeather.humidity({50.06, 19.94}) == &quot;Current humidity is 60%&quot; end endIn practice, you will have to ensure that during tests, the code you're testing uses the mock rather than the actual implementation. If the system under test relies on application configuration, you should set it before the test suite starts so that you can use async: true. You can do this in your config files:config :my_app, :weather_api, MyApp.MockWeatherAPIOr in your test_helper.exs:Application.put_env(:my_app, :weather_api, MyApp.MockWeatherAPI)All expectations are defined based on the current process. This means multiple tests using the same mock can still run concurrently unless the Mox is set to global mode. See the &quot;Multi-process collaboration&quot; section.Especially if you put the mock into the config/application environment you might want the implementation to fall back to a stub (or actual) implementation when no expectations are defined. stub_with/2 is just what you need! You can use it in your test (or in setup) as follows:Mox.stub_with(MyApp.MockWeatherAPI, MyApp.StubWeatherAPI)Now, if no expectations are defined it will call the implementation in MyApp.StubWeatherAPI.Multiple behavioursMox supports defining mocks for multiple behaviours.Suppose your library also defines a behaviour for getting past weather:defmodule MyApp.PastWeatherAPI do @callback past_temp(MyApp.LatLong.t(), DateTime.t()) :: {:ok, integer()} endYou can mock both the weather and past weather behaviour:Mox.defmock(MyApp.MockWeatherAPI, for: [MyApp.WeatherAPI, MyApp.PastWeatherAPI])Compile-time requirementsIf the mock needs to be available during the project compilation, for instance because you get undefined function warnings, then instead of defining the mock in your test_helper.exs, you should instead define it under test/support/mocks.ex:Mox.defmock(MyApp.MockWeatherAPI, for: MyApp.WeatherAPI)Then you need to make sure that files in test/support get compiled with the rest of the project. Edit your mix.exs file to add the test/support directory to compilation paths:def project do [ ... elixirc_paths: elixirc_paths(Mix.env), ... ] end defp elixirc_paths(:test), do: [&quot;test/support&quot;, &quot;lib&quot;] defp elixirc_paths(_), do: [&quot;lib&quot;]Multi-process collaborationMox supports multi-process collaboration via two mechanisms:explicit allowancesglobal modeThe allowance mechanism can still run tests concurrently while the global one doesn't. We explore both next.Explicit allowancesAn allowance permits a child process to use the expectations and stubs defined in the parent process while still being safe for async tests.test &quot;invokes add and mult from a task&quot; do MyApp.MockWeatherAPI |&gt; expect(:temp, fn _loc -&gt; {:ok, 30} end) |&gt; expect(:humidity, fn _loc -&gt; {:ok, 60} end) parent_pid = self() Task.async(fn -&gt; MyApp.MockWeatherAPI |&gt; allow(parent_pid, self()) assert MyApp.HumanizedWeather.temp({50.06, 19.94}) == &quot;Current temperature is 30 degrees&quot; assert MyApp.HumanizedWeather.humidity({50.06, 19.94}) == &quot;Current humidity is 60%&quot; end) |&gt; Task.await endNote: if you're running on Elixir 1.8.0 or greater and your concurrency comes from a Task then you don't need to add explicit allowances. Instead $callers is used to determine the process that actually defined the expectations.Global modeMox supports global mode, where any process can consume mocks and stubs defined in your tests. set_mox_from_context/0 automatically calls set_mox_global/1 but only if the test context doesn't include async: true.By default the mode is :private.setup :set_mox_from_context setup :verify_on_exit! test &quot;invokes add and mult from a task&quot; do MyApp.MockWeatherAPI |&gt; expect(:temp, fn _loc -&gt; {:ok, 30} end) |&gt; expect(:humidity, fn _loc -&gt; {:ok, 60} end) Task.async(fn -&gt; assert MyApp.HumanizedWeather.temp({50.06, 19.94}) == &quot;Current temperature is 30 degrees&quot; assert MyApp.HumanizedWeather.humidity({50.06, 19.94}) == &quot;Current humidity is 60%&quot; end) |&gt; Task.await endBlocking on expectationsIf your mock is called in a different process than the test process, in some cases there is a chance that the test will finish executing before it has a chance to call the mock and meet the expectations. Imagine this:test &quot;calling a mock from a different process&quot; do expect(MyApp.MockWeatherAPI, :temp, fn _loc -&gt; {:ok, 30} end) spawn(fn -&gt; MyApp.HumanizedWeather.temp({50.06, 19.94}) end) verify!() endThe test above has a race condition because there is a chance that the verify!/0 call will happen before the spawned process calls the mock. In most cases, you don't control the spawning of the process so you can't simply monitor the process to know when it dies in order to avoid this race condition. In those cases, the way to go is to &quot;sync up&quot; with the process that calls the mock by sending a message to the test process from the expectation and using that to know when the expectation has been called.test &quot;calling a mock from a different process&quot; do parent = self() ref = make_ref() expect(MyApp.MockWeatherAPI, :temp, fn _loc -&gt; send(parent, {ref, :temp}) {:ok, 30} end) spawn(fn -&gt; MyApp.HumanizedWeather.temp({50.06, 19.94}) end) assert_receive {^ref, :temp} verify!() endThis way, we'll wait until the expectation is called before calling verify!/0.","ref":"Mox.html","title":"Mox","type":"module"},{"doc":"Allows other processes to share expectations and stubs defined by owner process.ExamplesTo allow child_pid to call any stubs or expectations defined for MyMock:allow(MyMock, self(), child_pid)allow/3 also accepts named process or via references:allow(MyMock, self(), SomeChildProcess)","ref":"Mox.html#allow/3","title":"Mox.allow/3","type":"function"},{"doc":"Defines a mock with the given name :for the given behaviour(s).Mox.defmock(MyMock, for: MyBehaviour)With multiple behaviours:Mox.defmock(MyMock, for: [MyBehaviour, MyOtherBehaviour])Skipping optional callbacksBy default, functions are created for all the behaviour's callbacks, including optional ones. But if for some reason you want to skip one or more of its @optional_callbacks, you can provide the list of callback names to skip (along with their arities) as :skip_optional_callbacks:Mox.defmock(MyMock, for: MyBehaviour, skip_optional_callbacks: [on_success: 2])This will define a new mock (MyMock) that has a defined function for each callback on MyBehaviour except for on_success/2. Note: you can only skip optional callbacks, not required callbacks.You can also pass true to skip all optional callbacks, or false to keep the default of generating functions for all optional callbacks.Passing @moduledocYou can provide value for @moduledoc with :moduledoc option.Mox.defmock(MyMock, for: MyBehaviour, moduledoc: false) Mox.defmock(MyMock, for: MyBehaviour, moduledoc: &quot;My mock module.&quot;)","ref":"Mox.html#defmock/2","title":"Mox.defmock/2","type":"function"},{"doc":"Expects the name in mock with arity given by code to be invoked n times.If you're calling your mock from an asynchronous process and want to wait for the mock to be called, see the &quot;Blocking on expectations&quot; section in the module documentation.When expect/4 is invoked, any previously declared stub for the same name and arity will be removed. This ensures that expect will fail if the function is called more than n times. If a stub/3 is invoked after expect/4 for the same name and arity, the stub will be used after all expectations are fulfilled.ExamplesTo expect MockWeatherAPI.get_temp/1 to be called once:expect(MockWeatherAPI, :get_temp, fn _ -&gt; {:ok, 30} end)To expect MockWeatherAPI.get_temp/1 to be called five times:expect(MockWeatherAPI, :get_temp, 5, fn _ -&gt; {:ok, 30} end)To expect MockWeatherAPI.get_temp/1 not to be called:expect(MockWeatherAPI, :get_temp, 0, fn _ -&gt; {:ok, 30} end)expect/4 can also be invoked multiple times for the same name/arity, allowing you to give different behaviours on each invocation. For instance, you could test that your code will try an API call three times before giving up:MockWeatherAPI |&gt; expect(:get_temp, 2, fn _loc -&gt; {:error, :unreachable} end) |&gt; expect(:get_temp, 1, fn _loc -&gt; {:ok, 30} end) log = capture_log(fn -&gt; assert Weather.current_temp(location) == &quot;It&#39;s currently 30 degrees&quot; end) assert log =~ &quot;attempt 1 failed&quot; assert log =~ &quot;attempt 2 failed&quot; assert log =~ &quot;attempt 3 succeeded&quot; MockWeatherAPI |&gt; expect(:get_temp, 3, fn _loc -&gt; {:error, :unreachable} end) assert Weather.current_temp(location) == &quot;Current temperature is unavailable&quot;","ref":"Mox.html#expect/4","title":"Mox.expect/4","type":"function"},{"doc":"Chooses the Mox mode based on context.When async: true is used, set_mox_private/1 is called, otherwise set_mox_global/1 is used.Examplessetup :set_mox_from_context","ref":"Mox.html#set_mox_from_context/1","title":"Mox.set_mox_from_context/1","type":"function"},{"doc":"Sets the Mox to global mode.In global mode, mocks can be consumed by any process.An ExUnit case where tests use Mox in global mode cannot be async: true.Examplessetup :set_mox_global","ref":"Mox.html#set_mox_global/1","title":"Mox.set_mox_global/1","type":"function"},{"doc":"Sets the Mox to private mode.In private mode, mocks can be set and consumed by the same process unless other processes are explicitly allowed.Examplessetup :set_mox_private","ref":"Mox.html#set_mox_private/1","title":"Mox.set_mox_private/1","type":"function"},{"doc":"Allows the name in mock with arity given by code to be invoked zero or many times.Unlike expectations, stubs are never verified.If expectations and stubs are defined for the same function and arity, the stub is invoked only after all expectations are fulfilled.ExamplesTo allow MockWeatherAPI.get_temp/1 to be called any number of times:stub(MockWeatherAPI, :get_temp, fn _loc -&gt; {:ok, 30} end)stub/3 will overwrite any previous calls to stub/3.","ref":"Mox.html#stub/3","title":"Mox.stub/3","type":"function"},{"doc":"Stubs all functions described by the shared behaviours in the mock and module.Examplesdefmodule MyApp.WeatherAPI do @callback temp(MyApp.LatLong.t()) :: {:ok, integer()} @callback humidity(MyApp.LatLong.t()) :: {:ok, integer()} end defmodule MyApp.StubWeatherAPI do @behaviour WeatherAPI def temp(_loc), do: {:ok, 30} def humidity(_loc), do: {:ok, 60} end defmock(MyApp.MockWeatherAPI, for: MyApp.WeatherAPI) stub_with(MyApp.MockWeatherAPI, MyApp.StubWeatherAPI)This is the same as calling stub/3 for each callback in MyApp.MockWeatherAPI:stub(MyApp.MockWeatherAPI, :temp, &amp;MyApp.StubWeatherAPI.temp/1) stub(MyApp.MockWeatherAPI, :humidity, &amp;MyApp.StubWeatherAPI.humidity/1)","ref":"Mox.html#stub_with/2","title":"Mox.stub_with/2","type":"function"},{"doc":"Verifies that all expectations set by the current process have been called.","ref":"Mox.html#verify!/0","title":"Mox.verify!/0","type":"function"},{"doc":"Verifies that all expectations in mock have been called.","ref":"Mox.html#verify!/1","title":"Mox.verify!/1","type":"function"},{"doc":"Verifies the current process after it exits.If you want to verify expectations for all tests, you can use verify_on_exit!/1 as a setup callback:setup :verify_on_exit!","ref":"Mox.html#verify_on_exit!/1","title":"Mox.verify_on_exit!/1","type":"function"},{"doc":"","ref":"Mox.UnexpectedCallError.html","title":"Mox.UnexpectedCallError","type":"exception"},{"doc":"","ref":"Mox.VerificationError.html","title":"Mox.VerificationError","type":"exception"}]